/*
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef	_H_JFS_METAPAGE
#define _H_JFS_METAPAGE

#include <linux/pagemap.h>
#include <linux/jfs/jfs_lock.h>

typedef struct metapage {
	/* Common logsyncblk prefix (see jfs_logmgr.h) */
	u16 xflag;
	s16 lid;
	int lsn;
	 CDLL_ENTRY(logsyncblk) synclist;
	/* End of logsyncblk prefix */

	unsigned long flag;	/* See Below */
	unsigned long count;	/* Reference count */
	void *data;		/* Data pointer */

	/* list management stuff */
	struct metapage *hash_prev;
	struct metapage *hash_next;	/* Also used for free list */
	/*
	 * mapping & index become redundant, but we need these here to
	 * add the metapage to the hash before we have the real page
	 */
#ifdef kern22
	struct inode *mapping;
#else
	struct address_space *mapping;
#endif
	unsigned long index;
	wait_queue_head_t wait;
	struct list_head io_list;	/* I/O list */
	void (*end_io) (void *);	/* End-I/O function */
	void *endio_arg;	/* end_io argument */

	/* implementation */
	struct page *page;
	unsigned long logical_size;

	/* Journal management stuff to come... */
	int clsn;
	atomic_t nohomeok;
	struct jfs_log *log;
} metapage_t;

#ifndef kern22
/*
 * Direct-access address space operations
 */
extern struct address_space_operations direct_aops;
#endif	/* kern22 */

/* metapage flag */
#define META_locked	0
#define META_absolute	1
#define META_free	2
#define META_dirty	3
#define META_sync	4
#define META_discard	5
#define META_forced	6

#define mark_metapage_dirty(mp) set_bit(META_dirty, &(mp)->flag)

/* function prototypes */
extern metapage_t *__get_metapage(
				  struct inode *inode,
				  unsigned long lblock,
				  unsigned int size,
				  int absolute, unsigned long new);

#define read_metapage(inode, lblock, size, absolute)\
	 __get_metapage(inode, lblock, size, absolute, FALSE)

#define get_metapage(inode, lblock, size, absolute)\
	 __get_metapage(inode, lblock, size, absolute, TRUE)

extern void release_metapage(metapage_t *);

#define flush_metapage(mp) \
{\
	set_bit(META_dirty, &(mp)->flag);\
	set_bit(META_sync, &(mp)->flag);\
	set_bit(META_discard, &(mp)->flag);\
	release_metapage(mp);\
}

extern void __write_metapage(metapage_t *);

#ifdef kern22
#define __sync_metapage(mp)
#else
#define __sync_metapage(mp) \
	generic_buffer_fdatasync((struct inode *)mp->mapping->host,\
				 mp->page->index, mp->page->index + 1)
#endif

#define write_metapage(mp) \
{\
	set_bit(META_dirty, &(mp)->flag);\
	release_metapage(mp);\
}

#define discard_metapage(mp) \
{\
	clear_bit(META_dirty, &(mp)->flag);\
	set_bit(META_discard, &(mp)->flag);\
	release_metapage(mp);\
}

extern void hold_metapage(metapage_t *, int);

#endif				/* _H_JFS_METAPAGE */
